
# q-expansion of newform 650.2.j.f, downloaded from the LMFDB on 28 April 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 650
weight = 2
poly_data = [9, 0, -5, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0], 1], [[0, -2, 0, 1], 3], [[-3, 1, 1, 0], 1], [[3, 1, -1, 0], 1]]

hecke_ring_character_values = [[27, [0, -1, 0, 0]], [301, [0, -1, 0, 0]]]
aps_data = [[0, -1, 0, 0], [0, -1, 1, 0], [0, 0, 0, 0], [-3, 0, 0, 0], [1, -1, 2, 0], [3, 2, 0, 0], [2, 1, 1, 0], [2, 2, 0, 0], [-1, -1, 0, 2], [1, 3, 1, -1], [1, -1, 0, 0], [-3, 0, 0, 0], [-2, 4, 0, -2], [4, -1, 0, -3], [6, -1, 1, 1], [-5, -1, -4, 0], [-4, 2, 0, 2], [2, 0, 0, 0], [-3, 5, -3, 3], [-5, 4, 0, 1], [0, 4, 0, 0], [-3, -3, -3, 3], [-3, 1, -1, -1], [-4, 2, 0, 2], [3, -1, 3, -3], [-4, -6, -4, 4], [-5, -1, 0, 6], [1, -5, 0, 4], [2, 5, -3, 0], [-5, -7, 2, 0], [7, 7, 0, 0], [-3, 0, 0, 0], [-9, -1, 1, 1], [3, 6, 3, -3], [-13, -11, -2, 0], [1, -2, 3, 0], [-10, 10, 0, 0], [-3, 7, -3, 3], [-6, 2, -2, -2], [-1, -1, 0, 2], [15, 0, 0, 0], [-3, 3, -3, 3], [15, 3, -3, -3], [3, 1, 3, -3], [3, -6, 3, -3], [7, -3, 3, 3], [-10, 3, -3, -3], [-6, 3, -3, -3], [0, -18, 0, 0], [-10, 7, 0, 3], [-4, -1, 0, 5], [-13, -14, 1, 0], [16, 16, 0, 0], [-4, -6, -4, 4], [-13, -14, 1, 0], [10, 8, 2, 0], [4, 0, 4, -4], [-14, -17, 3, 0], [-8, 4, -12, 0], [-14, -16, 2, 0], [-5, -1, 0, 6], [6, 3, 6, -6], [6, -6, 6, 6], [-4, -6, -4, 4], [5, -4, 9, 0], [0, 12, 0, 0], [1, -1, 0, 0], [7, 10, -3, 0], [10, -5, 0, -5], [-1, -8, 0, 9], [-9, -5, 5, 5], [5, -13, 0, 8], [11, -5, 0, -6], [20, 20, 0, 0], [-13, -13, 0, 0], [-21, -2, 2, 2], [6, 6, -6, -6], [-18, 0, 0, 0], [1, 11, -10, 0], [17, 11, 6, 0], [1, -12, 1, -1], [4, -1, 0, -3], [10, -11, 0, 1], [19, -16, 0, -3], [-14, 6, -6, -6], [-5, 8, -13, 0], [-13, 17, 0, -4], [0, 2, 0, 0], [-11, 4, 0, 7], [-6, 10, -6, 6], [-13, -17, 4, 0], [-1, 2, 0, -1], [0, 2, 0, 0], [-1, 36, -1, 1], [2, -4, 6, 0], [10, 20, -10, 0], [-13, -11, -2, 0], [-9, -6, 6, 6], [5, 5, 0, 0], [1, -2, 3, 0], [-28, 25, 0, 3], [12, 5, -5, -5], [-4, -1, 0, 5], [3, -12, 12, 12], [6, 9, 6, -6], [-6, -22, -6, 6], [-3, -15, -3, 3], [-3, -3, -3, 3], [4, 0, 4, -4], [-16, -3, 3, 3], [-16, 10, 0, 6], [-24, 0, 0, 0], [0, 12, 0, 0], [-4, -8, 0, 12], [1, -2, 3, 0], [2, -12, 2, -2], [3, -3, 3, 3], [2, -2, 4, 0], [-20, -10, -10, 0], [-2, 6, -2, 2], [1, 13, -12, 0], [-8, -1, 0, 9], [7, -5, 0, -2], [3, 21, 3, -3], [-14, 14, 0, 0], [11, 9, 11, -11], [17, -23, 0, 6], [3, 3, -3, -3], [7, 1, 6, 0], [3, 12, -12, -12], [-10, 22, 0, -12], [9, -2, 2, 2], [6, -6, 6, -6], [5, -5, 0, 0], [-14, -4, -10, 0], [2, 8, -6, 0], [6, -5, 5, 5], [6, -6, 6, 6], [2, 10, -8, 0], [-2, 21, -2, 2], [10, -16, 0, 6], [-14, 19, 0, -5], [-2, 14, 0, -12], [18, -4, 4, 4], [1, -9, 9, 9], [-13, 17, 0, -4], [3, -14, 3, -3], [-13, -11, -2, 0], [-6, 0, -6, 6], [6, -5, 5, 5], [3, -16, 3, -3], [5, 0, 5, -5], [-5, 14, 0, -9], [-14, 10, 0, 4], [-8, 13, -21, 0], [-21, -3, 3, 3], [-6, 30, -6, 6], [2, -8, 10, 0], [11, -5, 0, -6], [22, -11, 0, -11], [-9, -1, 1, 1], [20, -1, 0, -19], [3, -46, 3, -3], [30, 3, -3, -3], [-3, 45, -3, 3], [-3, 12, -3, 3], [20, -12, 12, 12], [17, -5, 0, -12]]
